package me.unurled.sacredrealms.sr.components.entity;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import me.unurled.sacredrealms.sr.components.item.ItemStackDeserializer;
import me.unurled.sacredrealms.sr.components.item.ItemStackSerializer;
import me.unurled.sacredrealms.sr.components.player.PotionEffectDeserializer;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Mob;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.entity.EntitySpawnEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class EntityManager extends Manager {

  @NotNull private final HashMap<UUID, SREntity> entities;

  private final List<SREntityType> types;

  public EntityManager() {
    super();
    this.types = new ArrayList<>();
    entities = new HashMap<>();
  }

  /** Load the data */
  @Override
  public void loadData() {
    // loads all types
    DataManager dm = Manager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't load entity data.");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    if (dh == null) {
      error("DataHandler is null, Can't load entity data.");
      return;
    }

    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(PotionEffect.class, new PotionEffectDeserializer())
            .registerTypeAdapter(ItemStack.class, new ItemStackDeserializer())
            .excludeFieldsWithoutExposeAnnotation()
            .create();

    dh.getKeysAll("sr.entities")
        .forEach(
            key -> {
              SREntityType type = gson.fromJson(dh.get(key), SREntityType.class);
              if (type != null) {
                types.add(type);
              }
            });
  }

  public void saveType(@NotNull SREntityType type) {
    // save type
    DataManager dm = Manager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't save entity data.");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    if (dh == null) {
      error("DataHandler is null, Can't save entity data.");
      return;
    }
    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(PotionEffect.class, new PotionEffectDeserializer())
            .registerTypeAdapter(ItemStack.class, new ItemStackSerializer())
            .excludeFieldsWithoutExposeAnnotation()
            .create();
    dh.set("sr.entities." + type.getId(), gson.toJson(type));
  }

  @Nullable
  public SREntity getEntity(@NotNull UUID uuid) {
    return entities.get(uuid);
  }

  public void addEntity(@NotNull SREntity entity) {
    entities.put(entity.getUuid(), entity);
  }

  public void addEntity(@NotNull Entity e) {
    entities.put(e.getUniqueId(), new SREntity(e));
  }

  public boolean isSREntity(@NotNull UUID uuid) {
    return entities.containsKey(uuid);
  }

  public void addEntityType(@NotNull SREntityType type) {
    types.add(type);
  }

  public List<SREntityType> getTypes() {
    return types;
  }

  @Nullable
  public SREntityType getType(@NotNull String id) {
    for (SREntityType type : types) {
      if (type.getId().equals(id)) return type;
    }
    return null;
  }

  public List<SREntity> spawnEntity(
      @NotNull SREntityType type, int amount, double x, double y, double z, @NotNull World world) {
    // spawn entity
    List<SREntity> srEntities = new ArrayList<>();
    for (int i = 0; i < amount; i++) {
      Entity e = world.spawnEntity(new Location(world, x, y, z), type.getType());
      if (e instanceof Mob mob) {
        if (type.getHandItem() != null) {
          mob.getEquipment().setItemInMainHand(type.getHandItem());
        }
        if (!type.getArmor().isEmpty()) {
          mob.getEquipment().setArmorContents(type.getArmor().toArray(new ItemStack[0]));
        }
        if (type.getSecondHandItem() != null) {
          mob.getEquipment().setItemInOffHand(type.getSecondHandItem());
        }
      }
      SREntity srEntity = new SREntity(e);
      addEntity(srEntity);
      srEntities.add(srEntity);
    }

    return srEntities;
  }

  @EventHandler
  public void onEntitySpawn(@NotNull EntitySpawnEvent e) {
    if (e.getEntity() instanceof Player) return;

    if (isSREntity(e.getEntity().getUniqueId())) return;

    if (!(e.getEntity() instanceof Mob)) return;

    e.setCancelled(true);
  }
}
